#!/usr/local/bin/python
# -*- coding: UTF-8 -*-

#Copyright (C) 2006 by raptor
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# ordinary least squares
# author : raptor@we8log.com
# create : 2006/12/03
# revision : 2006/12/03

import random
import math
from pylab import *

def Sigma( func, size ) :
	c = 0
	for i in range(size) :
		c = c + func( i )
	return c

# ordinary least squares to Fourier
# Xlist and Ylist are some points
# restul is a matrix include cnSyntonicX(cnSyntonic+1) items
def ols2fourier( Xlist, Ylist, cnSyntonic ) :
	cnPoints = len( Ylist )
	line = []
	line.append( Sigma( lambda x : 0.5, cnPoints ) / 2 )
	for j in range(1, cnSyntonic) :
		line.append( Sigma( lambda x : math.cos( Xlist[x] * j * math.pi ) / 2, cnPoints ) )
	line.append( Sigma( lambda x : Ylist[x] / 2, cnPoints ) )
	result = []
	result.append( line )
	for n in range(1, cnSyntonic) :
		line = []
		line.append( Sigma( lambda x : math.cos( Xlist[x] * n * math.pi ), cnPoints ) / 2 )
		for j in range(1, cnSyntonic) :
			line.append( Sigma( lambda x : math.cos( Xlist[x] * j * math.pi ) * math.cos( Xlist[x] * n * math.pi ), cnPoints ) )
		line.append( Sigma( lambda x : Ylist[x] * math.cos( Xlist[x] * n * math.pi ), cnPoints ) )
		result.append( line )
	return result

# used by gauss function
# Ԫȥ
def mainitem( matrix, index ) :
	c = 0
	mi = 0
	for i in range( index, len( matrix ) ) :
		t = abs( matrix[i][index] )
		if t > c :
			c = t
			mi = i
	if mi != index :
		matrix[mi], matrix[index] = matrix[index], matrix[mi]

# GaussԪ
# matrixΪԷη
def gauss( matrix ) :
	cnSyntonic = len( matrix )
	for i in range( cnSyntonic - 1 ) :
		mainitem( matrix, i )
		for j in range( i + 1, cnSyntonic ) :
			k = -matrix[i][i] / matrix[j][i]
			matrix[j][i] = 0
			for n in range( i + 1, cnSyntonic + 1 ) :
				matrix[j][n] = matrix[j][n] * k + matrix[i][n]
	r = range( cnSyntonic )
	r.reverse()
	for i in r :
		for j in range( i + 1, cnSyntonic ) :
			matrix[i][cnSyntonic] = matrix[i][cnSyntonic] - matrix[i][j] * matrix[j][cnSyntonic]
		matrix[i][cnSyntonic] = matrix[i][cnSyntonic] / matrix[i][i]
	result = []
	for i in range( cnSyntonic ) :
		result.append( matrix[i][cnSyntonic] )
	return result

def funcvallist( Xlist, func ) :
	result = []
	for i in range( len( Xlist ) ) :
		result.append( func( Xlist[i] ) )
	return result

# Fourierֵ
def fourierval( coeff, Xlist ) :
	return funcvallist( Xlist, lambda xitem : coeff[0] / 2 + Sigma( lambda x : coeff[x + 1] * math.cos( xitem * ( x + 1 ) * math.pi ), len( coeff ) - 1 ) )

if __name__ == "__main__" :
	xsrc  = arange( 0, 1.001, 0.01 )
	ysrc = []
	for i in range(21) :
		ysrc.append( random.randint( 900, 1100 ) / 10.0 )
	points = []
	for i in range(20) :
		for j in range(5) :
			points.append( ( ysrc[i + 1] - ysrc[i] ) * j / 5 + ysrc[i] )
	points.append( ysrc[20] )
	plot( xsrc, points, "b" )
	matrix = ols2fourier( xsrc, points, 19 )
	#for i in range( len(matrix) ) :
	#	for j in range( len(matrix) + 1 ) :
	#		print matrix[i][j], "|", 
	#	print ""
	#print ""
	coefficient = gauss( matrix )
	#for i in range( len( coefficient ) ) :
	#	print coefficient[i], "|",
	#print ""
	ydest = fourierval( coefficient, xsrc )
	plot( xsrc, ydest, "g" )
	axis( [0, 1, 85, 120] )
	grid( True )

	diff = []
	for i in range( len( ydest ) ) :
		diff.append( ydest[i] - points[i] )
	a = axes( [ .4, .7, 0.2, 0.2 ], axisbg='y' )
	n, bins, patches = hist( diff, 19, normed=1 )
	setp( patches, 'facecolor', 'g', 'alpha', 0.75 )
	y = normpdf( bins, 0, 1)
	l = plot(bins, y)
	axis( [min(diff), max(diff), 0, max( [max(n), max(y)] ) ] )
	grid( True )

	show()
